# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,bigknn_knn_result)
S3method(as.data.frame,bigknn_radius_result)
S3method(print,bigknn_job)
S3method(print,bigknn_knn_result)
S3method(print,bigknn_plan)
S3method(print,bigknn_prepared)
S3method(print,bigknn_radius_result)
S3method(print,bigknn_recall)
S3method(summary,bigknn_knn_result)
S3method(summary,bigknn_plan)
S3method(summary,bigknn_prepared)
S3method(summary,bigknn_radius_result)
export(as_edge_list)
export(as_sparse_matrix)
export(as_triplet)
export(count_within_radius_bigmatrix)
export(knn_bigmatrix)
export(knn_graph_bigmatrix)
export(knn_graph_stream_bigmatrix)
export(knn_load_prepared)
export(knn_plan_bigmatrix)
export(knn_prepare_bigmatrix)
export(knn_search_prepared)
export(knn_search_stream_prepared)
export(knn_stream_bigmatrix)
export(knn_validate_prepared)
export(mutual_knn_graph_bigmatrix)
export(radius_bigmatrix)
export(radius_graph_bigmatrix)
export(radius_stream_bigmatrix)
export(radius_stream_job_bigmatrix)
export(recall_against_exact)
export(rerank_candidates_bigmatrix)
export(resume_knn_job)
export(snn_graph_bigmatrix)
importFrom(Rcpp,evalCpp)
useDynLib(bigKNN, .registration = TRUE)
